import React from 'react'

export default () => {

  return (
    <div className="header-wrapper">
      <header className="flex-row">
        <div className="">
          <button className="header-button">Forms</button>
          <button className="header-button">Records</button>
          <button className="header-button">Admin</button>
        </div>
      </header>
    </div>
  )
}
