const defaultNodes =  {
  "L9y4-xxI2Q": {
    id: "L9y4-xxI2Q",
    x: 912,
    y: 345.5,
    type: "output",
    width: 200,
    connections: { inputs: {}, outputs: {} },
    inputData: { label: { text: "" }, visible: {}, disabled: {}, required: {} }
  }
};

export default defaultNodes