:root {
  --input-background: linear-gradient(
    to bottom,
    #ededed 0%,
    #ffffff 35%,
    #ffffff 100%
  );
  --input-background-disabled: linear-gradient(
    to bottom,
    #c4c2c2 0%,
    #dedddd 15%,
    #dedddd 100%
  );
  --selected-hover-color: rgb(254, 250, 222);
  --selected-color: rgb(255, 248, 193);
}

.form-wrapper {
  display: flex;
  justify-content: center;
  font-family: "Roboto", sans-serif;
}
.form-sidebar {
  background: #fff;
  border-radius: 4px;
  box-shadow: 0px 15px 32px -4px rgba(0, 0, 0, 0.3);
  padding: 20px;
  display: flex;
  flex-direction: column;
  width: 250px;
}
.form-attributes {
  margin-left: 30px;
}
.form-sidebar-button {
  width: 100%;
  padding: 15px 5px;
  margin-bottom: 15px;
  background: rgb(238, 238, 238);
  border: none;
  border-radius: 3px;
  text-transform: uppercase;
  font-weight: 600;
  outline: none;
}
.form-sidebar-button:last-child {
  margin-bottom: 0px;
}
.form-sidebar-button:hover {
  background: rgb(217, 217, 217);
  cursor: grab;
}
form.form-body {
  width: 500px;
  min-height: 600px;
  margin-left: 30px;
  background: #fff;
  padding: 20px;
  border-radius: 4px;
  box-shadow: 0px 15px 32px -4px rgba(0, 0, 0, 0.3);
  display: flex;
  flex-direction: column;
}
.field-wrapper {
  padding: 15px;
  margin-left: -15px;
  margin-right: -15px;
  border-radius: 8px;
}
.field-wrapper:hover {
  background: var(--selected-hover-color);
}
.field-wrapper[data-selected="true"] {
  background: var(--selected-color);
}
.field-wrapper[data-selected="true"][data-visible="false"] {
  background: rgb(212, 207, 167);
}
.field-wrapper:first-child {
  margin-top: -10px;
}
.field-wrapper:last-child {
  margin-bottom: -10px;
}
.field-wrapper label {
  margin-bottom: 3px;
  font-size: 16px;
  font-weight: 500;
}
.field-wrapper[data-required="true"] label:not(.checkbox-box)::after {
  content: "*";
  margin-left: 5px;
  color: red;
}
.field-wrapper.unselectable {
  background: none;
}
.flex-column {
  display: flex;
  flex-direction: column;
}
.flex-row {
  display: flex;
  flex-direction: row;
}
.flex-wrap{
  flex-wrap: wrap;
}
.align-center {
  align-items: center;
}
.justify-center {
  justify-content: center;
}
.align-right {
  margin-left: auto;
}
.half-column {
  width: 50%;
}
.dummy-input {
  width: 300px;
  height: 40px;
  border: 1px solid rgb(176, 176, 176);
  border-radius: 4px;
  background: var(--input-background);
  position: relative;
  padding: 5px;
  padding-right: 20px;
}
.dummy-select::after {
  content: "";
  position: absolute;
  right: 8px;
  top: 15px;
  width: 0;
  height: 0;
  border-style: solid;
  border-width: 8px 6px 0 6px;
  border-color: #000000 transparent transparent transparent;
}
.dummy-checkbox-wrapper {
  display: flex;
  align-items: center;
}
.dummy-checkbox-wrapper label {
  padding-top: 3px;
}
.dummy-checkbox {
  width: 24px;
  height: 24px;
  border: 1px solid rgb(176, 176, 176);
  border-radius: 3px;
  background: var(--input-background);
  margin-right: 10px;
}
.real-checkbox-wrapper label {
  margin-bottom: 0px;
}
.real-checkbox-wrapper .checkbox-label {
  padding-top: 0px;
}
.attributes h2,
.form-sidebar h2,
.form-body h2 {
  margin: 0px;
  margin-bottom: 15px;
  cursor: default;
  border-radius: 5px;
}
.form-body h2:hover{
  background: var(--selected-hover-color);
  margin: -5px;
  margin-bottom: 10px;
  padding: 5px;
}
.form-body h2[data-editing="true"]{
  background: var(--selected-color);
  margin: -5px;
  margin-bottom: 10px;
  padding: 5px;
}
.attribute-wrapper {
  padding: 10px 0px;
  display: flex;
  flex-direction: column;
}
.attribute-wrapper:last-child {
  margin-bottom: -10px;
}
.attribute-wrapper label {
  font-size: 14px;
  font-weight: 500;
  margin-bottom: 3px;
  user-select: none;
}
.attribute-wrapper input,
.options-editor input {
  height: 33px;
  border: 1px solid rgb(176, 176, 176);
  border-radius: 4px;
  background: var(--input-background);
  padding: 8px;
}
.checkbox-wrapper {
  display: flex;
  align-items: center;
}
.checkbox-wrapper input,
.dummy-checkbox-wrapper input {
  width: 1px;
  height: 1px;
  opacity: 0.01;
  background: transparent;
  border: none;
  position: absolute;
  pointer-events: none;
  left: 0px;
  top: 0px;
}
.checkbox-box {
  width: 24px;
  height: 24px;
  border: 1px solid rgb(176, 176, 176);
  border-radius: 3px;
  background: var(--input-background);
  margin-right: 10px;
  display: flex;
  justify-content: center;
  align-items: center;
  text-align: center;
  padding-left: 4px;
}
.checkbox-wrapper input[value="true"] + .checkbox-box::after,
.real-checkbox-wrapper input[value="true"] + .checkbox-box::after {
  content: "✔️";
  font-size: 18px;
}
.real-checkbox-wrapper .checkbox-box {
  padding: 2px;
  padding-left: 4px;
  padding-right: 0px;
}
.field-wrapper [data-disabled="true"],
.field-wrapper [disabled],
.field-wrapper [disabled] + .checkbox-box {
  background: var(--input-background-disabled);
}
.field-wrapper[data-visible="false"] {
  background: rgb(222, 222, 222);
  position: relative;
}
.field-wrapper[data-visible="false"]::after {
  content: "👁";
  position: absolute;
  right: 7px;
  top: 7px;
  font-size: 18px;
}
.field-wrapper label{
  user-select: none;
}
.modal-wrapper {
  position: fixed;
  left: 0px;
  top: 0px;
  width: 100%;
  height: 100%;
  display: flex;
  justify-content: center;
  align-items: center;
  z-index: 12;
}
.modal-shade {
  position: fixed;
  left: 0px;
  top: 0px;
  width: 100%;
  height: 100%;
  background: rgba(0, 0, 0, 0.2);
  z-index: 9;
}
.modal-body {
  background: #fff;
  border-radius: 10px;
  box-shadow: 0px 15px 32px -4px rgba(0, 0, 0, 0.3);
  padding: 20px;
  z-index: 99;
}
.modal-body h2 {
  margin: 0px;
  padding-top: 5px;
}
button.attribute-button {
  padding: 10px 15px;
  border-radius: 4px;
  border: none;
  background: linear-gradient(to bottom, rgb(55, 114, 255), rgb(39, 88, 204));
  color: #fff;
  text-transform: uppercase;
  font-weight: 500;
  font-family: "Roboto", sans-serif;
  outline: none;
}
button.attribute-button:hover {
  background: linear-gradient(to bottom, rgb(82, 133, 255), rgb(48, 99, 219));
}
button.attribute-button:active{
  background: linear-gradient(to top, rgb(55, 114, 255), rgb(39, 88, 204));
}
button.attribute-button.secondary {
  background: linear-gradient(
    to bottom,
    rgb(221, 221, 221),
    rgb(201, 201, 201)
  );
  color: #000;
}
button.attribute-button.secondary:hover {
  background: linear-gradient(
    to bottom,
    rgb(235, 235, 235),
    rgb(215, 215, 215)
  );
}
button.attribute-button.secondary:active {
  background: linear-gradient(
    to top,
    rgb(235, 235, 235),
    rgb(215, 215, 215)
  );
}
button.attribute-button.danger {
  background: linear-gradient(
    to bottom,
    rgb(231, 45, 45),
    rgb(184, 21, 21)
  );
  color: #fff;
}
button.attribute-button.danger:hover {
  background: linear-gradient(
    to bottom,
    rgb(242, 64, 64),
    rgb(201, 26, 26)
  );
}
button.attribute-button.danger:active {
  background: linear-gradient(
    to top,
    rgb(212, 35, 35),
    rgb(199, 32, 32)
  );
}
button.attribute-button.small {
  font-size: 14px;
  padding: 8px 12px;
}
.options-editor input {
}
.options-editor input:last-child {
  margin-left: 15px;
}
.options-editor .half-column:last-child label {
  margin-left: 10px;
}
.options-editor .controls-row {
  margin-bottom: 15px;
}
.options-editor .add-button {
  padding: 10px 15px;
  background: rgb(230, 230, 230);
  border-radius: 4px;
  text-transform: uppercase;
  font-size: 16px;
  font-weight: 500;
  border: none;
  font-family: "Roboto", sans-serif;
}
.options-editor .add-button:hover {
  background: rgb(240, 237, 237);
}
.options-editor .controls-footer {
  margin-top: 15px;
}
.options-editor .option-row {
  margin-bottom: 15px;
}
.options-editor .option-row:last-of-type {
  margin-bottom: 0px;
}
.form-sidebar[data-previewing="true"] {
  opacity: 0.4;
  pointer-events: none;
}
.header-wrapper{
  display: flex;
  justify-content: center;
  margin-top: -45px;
  margin-bottom: 30px;
}
.header-wrapper header{
  width: 100%;
  padding: 10px 10px;
  opacity: .4;
  transition: opacity 400ms;
}
.header-wrapper header:hover{
  opacity: 1;
}
button.header-button{
  padding: 8px 15px;
  border: none;
  background: rgba(255,255,255,.3);
  border-radius: 5px;
  margin: 0px 5px;
  font-family: "Roboto", sans-serif;
  font-weight: 500;
  text-transform: uppercase;
  font-size: 14px;
  color: rgb(58, 69, 89);
}
button.header-button:hover{
  background: rgba(255,255,255,.6);
}
button.header-button:last-child{
  margin-right: 0px;
}
.page-wrapper{
  width: 100%;
}
.floating-nav{
  position: fixed;
  right: 30px;
  bottom: 30px;
  border-radius: 40px;
  background: #fff;
  box-shadow: 0px 15px 32px -4px rgba(0, 0, 0, 0.3);
  width: 60px;
  height: 60px;
  transition: height 400ms;
  z-index: 9;
  overflow: hidden;
}
.floating-nav-menu{
  position: absolute;
  left: 0px;
  bottom: 40px;
  width: 100%;
  height: 200px;
  display: flex;
  flex-direction: column;
  justify-content: center;
  align-items: center;
}
.floating-nav-menu a{
  width: 60px;
  height: 60px;
  display: flex;
  justify-content: center;
  align-items: center;
}
.floating-nav-menu a svg{
  width: 30px;
  height: 30px;
}
.floating-nav-exit{
  position: fixed;
  width: 200px;
  height: 360px;
  right: 0px;
  bottom: 0px;
  z-index: 8;
}
.nav-icon-wrapper{
  display: flex;
  flex-direction: column;
  width: 60px;
  height: 60px;
  justify-content: space-between;
  position: absolute;
  left: 0px;
  bottom: 2px;
  padding: 15px;
  padding-top: 20px;
  pointer-events: none;
  transition: transform 300ms;
}
.floating-nav-menu a:hover svg{
  fill: rgb(55, 83, 131);
}
.nav-icon-wrapper .bar{
  width: 100%;
  border-radius: 4px;
  height: 3px;
  background: #000;
  transition: transform 300ms, opacity 300ms, width 300ms;
}
.nav-icon-wrapper .bar:first-child{
  transform-origin: left center;
}
.nav-icon-wrapper .bar:last-child{
  transform-origin: left center;
}
.nav-icon-wrapper[data-menu-open="true"]{
  transform: scaleX(.85);
}
.nav-icon-wrapper[data-menu-open="true"] .bar:first-child{
  transform: rotate(37deg);
  width: calc(100% + 7px);
}
.nav-icon-wrapper[data-menu-open="true"] .bar:nth-child(2){
  opacity: 0;
}
.nav-icon-wrapper[data-menu-open="true"] .bar:last-child{
  transform: rotate(-37deg);
  width: calc(100% + 7px);
}
.hover-label{
  position: fixed;
  right: 100px;
  bottom: 0px;
  background: #fff;
  box-shadow: 0px 15px 32px -4px rgba(0, 0, 0, 0.3);
  padding: 7px 15px;
  border-radius: 100px;
  font-family: "Roboto", sans-serif;
  opacity: 0;
  transition: opacity 300ms, transform 300ms;
}
.hover-label[data-show-label="true"]{
  opacity: 1;
  transform: translateX(-10px);
}
