.wrapper{
  background: rgba(91, 96, 99, 0.9);
  border-radius: 5px;
  box-shadow: 0px 4px 8px rgba(0,0,0,0);
  position: absolute;
  left: 0px;
  top: 0px;
  user-select: none;
  display: flex;
  flex-direction: column;
  z-index: 1;
  cursor: default;
}
.label{
  font-size: 13px;
  text-transform: uppercase;
  padding: 5px;
  background: #464b4e;
  border-radius: 5px 5px 0px 0px;
  margin: 0px;
  margin-bottom: 3px;
  border-bottom: 1px solid rgba(0,0,0,.15);
}
