.wrapper{
  display: flex;
  align-items: center;
}
.checkbox{
  background: linear-gradient(to bottom, #5b5f62, #6f7477);
  border: 1px solid #3c3e40;
  border-radius: 4px;
  margin-right: 8px;
}
.label{
  padding-top: 2px;
  font-size: 13px;
}
