/* stylelint-disable docusaurus/copyright-header */
/**
 * Any CSS included here will be global. The classic template
 * bundles Infima by default. Infima is a CSS framework designed to
 * work well for content-centric websites.
 */

/* You can override the default Infima variables here. */
@import url("https://use.typekit.net/nge5ixu.css");

:root {
  --ifm-color-primary: #00a27b;
  --ifm-color-primary-dark: rgb(33, 175, 144);
  --ifm-color-primary-darker: rgb(31, 165, 136);
  --ifm-color-primary-darkest: rgb(26, 136, 112);
  --ifm-color-primary-light: rgb(70, 203, 174);
  --ifm-color-primary-lighter: rgb(102, 212, 189);
  --ifm-color-primary-lightest: rgb(146, 224, 208);
  --ifm-code-font-size: 95%;
  --blue-color: #2F6AEC;
  --blue-light-color: #3c76f6;
  --green-color: #30D673;
  --green-light-color: #44e886;
  --red-color: #F23D63;
  --red-light-color: #fc5175;
  --primary-font: "Paralucent", sans-serif;
  --background-dark: #1C1C1C;
  --background-light: #262626;
  --background-extra-light: #2e2e2e;
}

.docusaurus-highlight-code-line {
  background-color: rgb(72, 77, 91);
  display: block;
  margin: 0 calc(-1 * var(--ifm-pre-padding));
  padding: 0 var(--ifm-pre-padding);
}

.navbar__logo{
  height: 50%;
}
.docusaurus-highlight-code-line{
  background-color: rgb(29, 32, 37);
}
