const sidebar = {
  someSidebar: {
    Concepts: ["overview", "anatomy", "node-editor", "type-safety", "running-logic", "faq"],
    "Guides": [
      "quick-start",
      "basic-config",
      "root-node",
      "logic-nodes",
      "saving-nodes",
      "root-engine",
      "using-with-react",
      "using-without-react",
      "dynamic-nodes",
      "comments",
      "theming"
    ],
    API: ["NodeEditor", "flume-config", "RootEngine", "controls", "colors"]
  }
};

module.exports = sidebar