.wrapper{
  background: none;
  border: none;
  /* float: left; */
  flex: 1;
}
.input{
  background: linear-gradient(to bottom, #5b5f62, #6f7477);
  width: 100%;
  height: 100%;
  border: 1px solid #3c3e40;
  border-radius: 4px;
  font-size: 13px;
  padding-top: 2px;
  resize: vertical;
  outline: none;
  &::placeholder{
    color: rgb(47, 49, 50);
  }
  &:focus{
    background: linear-gradient(to bottom, #676b6e, #75797c);
  }
}
