/* stylelint-disable docusaurus/copyright-header */

/**
 * CSS files with the .module.css suffix will be treated as CSS modules
 * and scoped locally.
 */
.mobile,
.mobileInline {
  display: none;
}
.mobileSmallInline {
  display: none;
}
.homePageWrapper {
  display: flex;
  flex-direction: column;
  align-items: center;
  font-family: var(--primary-font);
  color: #fff;
}
.homeHeader {
  display: flex;
  width: 100%;
  padding: 20px;
  font-family: var(--primary-font);
}
.alignRight {
  margin-left: auto;
}
.logo {
  width: 180px;
}
.navLinks {
  margin-left: 40px;
  height: 100%;
  display: flex;
  align-items: center;
}
.navLinks a {
  color: #fff;
  font-size: 20px;
}
.heroBlock {
  display: flex;
  width: 100%;
  min-height: calc(100vh - 120px);
  background: var(--background-dark);
}
.heroTextBlock {
  display: flex;
  justify-content: center;
  align-items: center;
  flex-direction: column;
  width: 55%;
  padding-bottom: 50px;
}
.heroTextInnerWrapper {
  display: flex;
  flex-direction: column;
}
.wordmark {
  max-width: 50px;
}
.heroTextBlock h1 {
  display: flex;
  flex-direction: column;
  font-weight: 800;
  font-family: "Paralucent", sans-serif;
  font-size: 110px;
  line-height: 0.95;
  margin-bottom: 0px;
}
.heroTextBlock .heroSubtitle {
  color: #606060;
  font-size: 45px;
  margin-top: 15px;
  line-height: 0.6;
}
.heroTextBlock .heroSubSubtitle {
  font-size: 28px;
  line-height: 1.2;
  margin-top: 0px;
}
.blueText {
  color: var(--blue-color);
}
.greenText {
  color: var(--green-color);
}
.redText {
  color: var(--red-color);
}
.redText a:hover {
  color: var(--red-light-color);
}
.redStroke path {
  stroke: var(--red-color);
}
.whiteText {
  color: #fff;
}
.inlineCtaLink {
  color: inherit;
  height: 1em;
  display: inline-block;
  margin-top: 15px;
}
.inlineCtaLink:hover {
  color: inherit;
  text-decoration: none;
}
.inlineCtaLink:hover svg {
  transform: translateX(5px);
}
.inlineCtaLink svg {
  width: 0.9em;
  margin-left: 0.3em;
  transition: transform 150ms;
}
.heroImageBlock {
  height: 100%;
  flex: 1 1 auto;
  display: flex;
}
.heroImageBlock img.heroNodes {
  height: calc(100% - 300px);
  position: absolute;
  right: 0px;
  top: 150px;
}
.videoCtaWrapper {
  display: flex;
  flex-direction: column;
  width: 100%;
  position: relative;
  margin-top: -100px;
  margin-bottom: -150px;
}
.videoCtaInnerWrapper {
  background: #3e3e3e;
  width: 100%;
  max-height: 500px;
}
.videoPreview {
  width: 100%;
  opacity: 0.8;
}
.videoCurve {
  transform: rotate(180deg);
  position: absolute;
  left: 0px;
  top: -1px;
  width: 100%;
  margin: 0px;
  z-index: 9;
}
.videoButtonWrapper {
  width: 100%;
  height: 100%;
  position: absolute;
  left: 0px;
  top: 0px;
  display: flex;
  align-items: center;
  padding-left: 60px;
}
.playVideoButton {
  background: none;
  border: none;
  display: flex;
  align-items: center;
  line-height: 0;
  border-radius: 100px;
  padding: 5px;
  padding-right: 15px;
  font-size: 40px;
  font-family: "Paralucent", sans-serif;
  font-weight: 400;
  margin-top: -60px;
  color: #fff;
  z-index: 10;
  opacity: 0.8;
  text-shadow: 0px 2px 1px rgba(0, 0, 0, 0.8);
}
.playVideoButton:hover {
  opacity: 1;
}
.playVideoButton img {
  width: 80px;
  height: 80px;
  margin-right: 30px;
}
.videoWrapper {
  position: fixed;
  left: 0px;
  top: 0px;
  width: 100%;
  height: 100%;
  display: flex;
  justify-content: center;
  align-items: center;
  z-index: 99;
  padding: 50px;
}
.videoShade {
  position: absolute;
  left: 0px;
  top: 0px;
  width: 100%;
  height: 100%;
  background: rgba(0, 0, 0, 0.6);
  z-index: 5;
}
.videoInnerWrapper {
  position: absolute;
  left: 0px;
  top: 0px;
  width: 100%;
  height: 100%;
  display: flex;
  justify-content: center;
  align-items: center;
  z-index: 9;
}
.embedContainer {
  position: relative;
  padding-bottom: 56.25%;
  height: 0;
  overflow: hidden;
  max-width: 100%;
  width: 100%;
  z-index: 99;
  border-radius: 8px;
}
.embedContainer iframe,
.embedContainer object,
.embedContainer embed {
  position: absolute;
  top: 0;
  left: 0;
  width: 100%;
  height: 100%;
}
.license {
  margin-left: 25px;
  color: rgb(96, 96, 96);
  font-size: 20px;
}
.pillButton {
  border-radius: 100px;
  background: var(--blue-color);
  color: #fff;
  padding: 10px 20px;
  font-size: 24px;
  border: none;
  font-family: var(--primary-font);
  white-space: nowrap;
}
.pillButton:hover {
  text-decoration: none;
  background: var(--blue-light-color);
  color: #fff;
}
.pillButton:hover svg {
  text-decoration: none;
  background: var(--blue-light-color);
  color: #fff;
  transform: translateX(5px);
}
.pillButton svg {
  width: 24px;
  margin-left: 7px;
  transition: transform 150ms;
}
.pillSmall {
  font-size: 18px;
  padding: 8px 20px;
}
.pillWhite {
  background: #fff;
  color: var(--background-dark);
}
.pageCurve {
  width: 100%;
  margin-top: -100%;
}
.exampleBlock {
  width: 100%;
  margin-top: 50px;
  background: var(--background-light);
  padding-bottom: 30vh;
  position: relative;
  z-index: 6;
}
.exampleBlock .pageCurve {
  transform: rotateY(180deg);
}
.exampleInnerWrapper {
  padding: 0px 8vw;
}
.exampleTextBlock {
  display: flex;
  flex-direction: column;
  width: 100%;
}
.exampleTitleBlock {
  display: flex;
  flex-direction: column;
  font-size: 80px;
  line-height: 1.1;
}
.exampleTextBlock p {
  font-size: 35px;
  line-height: 1.4;
}
.flumeWrapper {
  width: 100%;
  height: 60vh;
  border-radius: 40px;
  background: var(--background-dark);
  overflow: hidden;
  margin-top: 20px;
  color: #000;
}
.flumeWrapper p {
  margin: 0;
  padding: 0;
}
.darkFill {
  background: var(--background-dark);
}
.typeSafeWrapper {
  width: 100%;
  padding-bottom: 20vh;
  background-color: var(--background-dark);
  z-index: 9;
}
.typeSafeInnerWrapper {
  display: flex;
  flex-direction: row;
  padding: 0px 8vw;
  /* padding-right: 6vw; */
}
.halfTextBlock {
  width: 60%;
  display: flex;
  justify-content: center;
  align-items: center;
}
.typeSafeImageWrapper {
  width: 45%;
  max-width: 600px;
  transform: translate(7%, 6%);
  display: flex;
  align-items: center;
  margin-top: -15vh;
}
.typeSafeImage {
  margin-top: 4%;
  width: 100%;
  height: 100%;
  max-width: 500px;
}
.innerTextWrapper {
  max-width: 700px;
}
.featuresBlock {
  width: 100%;
  background: var(--background-light);
  padding-bottom: 20vh;
  z-index: 10;
}
.featuresBlockInnerWrapper {
  padding: 0px 8vw;
  padding-top: 8vh;
  padding-bottom: 10vh;
}
.featureTile {
  display: flex;
  margin-bottom: 40px;
}
.featureTile:last-child {
  margin-bottom: 0px;
}
.featureImageWrapper {
  width: 130px;
  flex-shrink: 0;
}
.featureImageWrapper img {
  width: 100%;
}
.featureTextColumn {
  display: flex;
  flex-direction: column;
  padding-left: 30px;
}
.featureTitle {
  font-size: 40px;
}
.featureDescription {
  font-size: 24px;
  line-height: 1.2;
}
.runLogicWrapper {
  width: 100%;
  background: var(--background-dark);
  padding-bottom: 25vh;
  z-index: 11;
}
.flexColumn {
  display: flex;
  flex-direction: column;
}
.centerFlex {
  display: flex;
  justify-content: center;
  align-items: center;
  flex-direction: column;
}
.callToActionWrapper {
  width: 100%;
  margin-top: -20px;
  background: var(--background-light);
  padding-bottom: 10vh;
  z-index: 13;
}
.ctaWordmark {
  width: 200px;
}
.ctaInnerWrapper {
  padding: 3vh 0px;
}
.ctaInnerWrapper p {
  font-size: 30px;
  margin-top: 40px;
  margin-bottom: 30px;
}
.callToActionWrapper .pageCurve {
  margin-top: calc(-100% + 20px);
}
.wordmarkLeft {
  fill: var(--blue-color);
  animation: rotate-fill 900ms linear infinite;
}
.wordmarkRight {
  fill: var(--green-color);
  animation: rotate-fill 900ms linear infinite;
  animation-delay: -450ms;
}
.footer{
  width: 100%;
  display: flex;
  align-items: center;
  background: var(--background-extra-light);
  padding: 30px;
}
.netlifyWrapper{
  padding: 10px 15px;;
  border-radius: 9px;
  background: rgba(0,0,0,.2);
  display: flex;
  justify-content: center;
  align-items: center;
}
.netlifyLogo{
  width: 80px;
  margin: 0px;
}
.netlifyRecommend{
  display: flex;
  align-items: center;
}
.hostedBy{
  margin-right: 10px;
  text-align: center;
}
@keyframes rotate-fill {
  0% {
    fill: var(--blue-color);
  }
  33.3333% {
    fill: var(--green-color);
  }
  66.6666% {
    fill: var(--red-color);
  }
  100% {
    fill: var(--blue-color);
  }
}

@media screen and (max-height: 750px){
  .heroBlock{
    min-height: calc(100vh - 30px);
    padding-bottom: 15vh;
  }
  .heroImageBlock img.heroNodes{
    min-height: 400px;
  }
}

@media screen and (max-width: 1300px) {
  .heroImageBlock img.heroNodes {
    max-width: 600px;
  }
}

@media screen and (max-width: 1200px) {
  .heroImageBlock img.heroNodes {
    max-width: 500px;
    top: 100px;
  }
  .heroTextInnerWrapper h1 {
    font-size: 90px;
  }
  .heroTextBlock .heroSubtitle {
    font-size: 35px;
  }
  .heroTextBlock .heroSubSubtitle {
    font-size: 23px;
  }
  .heroBlock {
    min-height: calc(100vh - 250px);
  }
  .exampleTitleBlock {
    font-size: 60px;
  }
  .exampleTextBlock p {
    font-size: 28px;
  }
}

@media screen and (max-width: 960px) {
  .logo {
    width: 150px;
  }
  .navLinks {
    margin-left: 20px;
  }
  .desktop {
    display: none;
  }
  .mobile {
    display: inline-block;
  }
  .mobileInline {
    display: inline;
  }
  .heroBlock{
    padding-bottom: 0px;
  }
  .heroImageBlock {
    position: relative;
    width: 100%;
    display: flex;
    justify-content: flex-end;
    padding-left: 30px;
    padding-top: 80px;
    padding-bottom: 100px;
  }
  .heroImageBlock img.heroNodes {
    position: relative;
    width: 124%;
    height: 100%;
    margin-left: -24%;
    height: auto;
    max-width: none;
    top: 0px;
  }
  .heroTextInnerWrapper {
    padding-left: 15px;
    padding-right: 15px;
  }
  .heroTextInnerWrapper h1 {
    font-size: calc(20px + 15vw);
    text-align: center;
    padding-left: 10px;
  }
  .heroTextBlock {
    width: 100%;
    min-height: 75vh;
    padding-bottom: 0px;
  }
  .heroTextBlock .heroSubtitle {
    font-size: 30px;
    text-align: center;
    line-height: 1;
  }
  .heroTextBlock .heroSubSubtitle {
    font-size: 24px;
  }
  .heroBlock {
    flex-direction: column;
    min-height: auto;
    padding-top: 50px;
  }
  .exampleBlock {
    padding-bottom: 12vh;
    margin-top: 0px;
  }
  .exampleTitleBlock {
    font-size: 55px;
    display: inline-block;
  }
  .exampleTextBlock p {
    font-size: 24px;
  }
  .flumeWrapper {
    pointer-events: none;
    height: 35vh;
    border-radius: 25px;
  }
  .typeSafeWrapper {
    padding-bottom: 6vh;
  }
  .typeSafeInnerWrapper {
    flex-direction: column;
    align-items: center;
  }
  .halfTextBlock {
    width: 100%;
  }
  .typeSafeImageWrapper {
    width: 100%;
    transform: none;
    justify-content: center;
    max-width: 500px;
    margin-top: -8vh;
    margin-bottom: 3vw;
  }
  .typeSafeImage {
    margin-top: 0px;
  }
  .mobileColumnCenter {
    display: flex;
    flex-direction: column;
    align-items: center;
  }
  .license {
    margin-left: 0px;
    margin-top: 10px;
  }
  .featuresBlock {
    padding-bottom: 5vh;
  }
}
@media screen and (max-width: 800px) {
  .featureImageWrapper {
    width: 110px;
  }
}
@media screen and (max-width: 600px) {
  .mobileSmallInline {
    display: none;
  }
  .featureTile {
    flex-direction: column;
    align-items: center;
  }
  .featureTextColumn {
    padding-left: 0px;
  }
  .featureTitle {
    text-align: center;
    font-size: 34px;
  }
  .featureDescription {
    font-size: 22px;
    line-height: 1.4;
  }
  .flumeWrapper {
  }
  .netlifyRecommend{
    flex-direction: column;
  }
  .netlifyRecommend .hostedBy{
    margin-right: 0px;
    margin-bottom: 10px;
  }
}
@media screen and (max-width: 400px){
  .footer{
    flex-direction: column-reverse;
  }
  .netlifyRecommend{
    margin-left: 0px;
    margin-bottom: 20px;
  }
}
