.wrapper {
  width: 100%;
  height: 100%;
  min-height: 100px;
  background-color: #292C33;
  /* background-image: linear-gradient(
      0deg,
      transparent 24%,
      rgba(255, 255, 255, 0.04) 25%,
      rgba(255, 255, 255, 0.04) 26%,
      transparent 27%,
      transparent 74%,
      rgba(255, 255, 255, 0.04) 75%,
      rgba(255, 255, 255, 0.04) 76%,
      transparent 77%,
      transparent
    ),
    linear-gradient(
      90deg,
      transparent 24%,
      rgba(255, 255, 255, 0.04) 25%,
      rgba(255, 255, 255, 0.04) 26%,
      transparent 27%,
      transparent 74%,
      rgba(255, 255, 255, 0.04) 75%,
      rgba(255, 255, 255, 0.04) 76%,
      transparent 77%,
      transparent
    ); */
  color: #000;
  background-size: 30px 30px;
  position: relative;
  overflow: hidden;
  -webkit-overflow-scrolling: touch;
  font-family: Helvetica, sans-serif;
  text-align: left;
  line-height: 1;
  outline: none !important;
}
.wrapper * {
  box-sizing: border-box;
}
.wrapper input,
textarea,
select {
  font-family: Helvetica, sans-serif;
}
.transformWrapper {
  transform-origin: center center;
  position: absolute;
  left: 50%;
  top: 50%;
  width: 0px;
  height: 0px;
}
.scaleWrapper {
  position: absolute;
  left: 0px;
  top: 0px;
  width: 0px;
  height: 0px;
}
