*{
  box-sizing: border-box;
}
body {
  margin: 0;
  padding: 0;
  font-family: sans-serif;
  background: rgb(17, 18, 19);
}
.wrapper{
  width: 1600px;
  height: 900px;
}
#root{
  display: flex;
  justify-content: center;
  align-items: center;
  width: 100%;
  padding-top: 50px;
}
.website-wrapper{
  color: #fff;
}
